input yearInfoForProj[yearInfoSet]   !integer
input boundsChecking[boundsInfoSet]  !real
input normInfo[normInfoSet] !integer
input percentChgFromLYHis[] !real between 0 and 100
input defaultJoin[] ! put 1 for join projection of the trend and 2 for join projection of the last 2 years of history and 3 to join the trend projection of the last year of history
input varNameHis[upperdims,timeseq]
output varNameResult[upperdims,timeseq]
output firstHisYearForProjection[]
output asymtoticPercentChange[]
output joinMethod[]


! NOTE these informants should be put into your model they are just here for reference
skipto afterInformants
localinformant yearInfoSet[] = create (; object=set, \
	desc="defaults wrt projection year", \
	elemName1 = "min", \
	elemName2 = "max", \
	elemName3 = "default", \
	elemName4 = "timeDimNum")
localinformant boundsInfoSet[] = create (; object=set, \
	desc="defaults wrt bounds checking", \
	elemName1 = "setMax", \
	elemName2 = "maxVal", \
	elemName3 = "setMin", \
	elemName4 = "minVal")
localinformant normInfoSet[] = create (; object=set, \
	desc="defaults wrt normalizing", \
	elemName1 = "doRenorm", \
	elemName2 = "normDimNum") 		
afterInformants:

! Helper code for preparing to call this function:

!			local yearInfoForProj[yearInfoSet] = create (; dim=yearInfoSet)
!			yearInfoForProj[yearInfoSet] = insert (; yearInfoSet:min)
!			yearInfoForProj[yearInfoSet] = insert (; yearInfoSet:max)
!			yearInfoForProj[yearInfoSet] = insert (; yearInfoSet:default)


integer $minYearToStartProj, $maxYearToStartProj, $defaultYearToStartProj, $timeDimNum
getobjinfo (yearInfoForProj[yearInfoSet], $minYearToStartProj; info=dataElement, dim1=min)
getobjinfo (yearInfoForProj[yearInfoSet], $maxYearToStartProj; info=dataElement, dim1=max)
getobjinfo (yearInfoForProj[yearInfoSet], $defaultYearToStartProj; info=dataElement, dim1=default)
getobjinfo (yearInfoForProj[yearInfoSet], $timeDimNum; info=dataElement, dim1=timeDimNum)


integer $firstHisYear, $lastHisYear, $firstScnYear, $lastScnYear
getobjinfo (varNameHis[upperdims,timeseq], $firstHisYear; info=sequenceStart, dim=$timeDimNum)
getobjinfo (varNameHis[upperdims,timeseq], $lastHisYear; info=sequenceEnd, dim=$timeDimNum)
getobjinfo (varNameResult[upperdims,timeseq], $firstScnYear; info=sequenceStart, dim=$timeDimNum)
getobjinfo (varNameResult[upperdims,timeseq], $lastScnYear; info=sequenceEnd, dim=$timeDimNum)

integer $lastHisYearP1
$lastHisYearP1 = $lastHisYear + 1

integer $setMax
real $maxVal
boolean $doMax = false
getobjinfo (boundsChecking[], $setMax; info=dataElement, dim1=setMax)
getobjinfo (boundsChecking[], $maxVal; info=dataElement, dim1=maxVal)
if ($setMax == 1)
	$doMax = true
endif

integer $setMin
real $minVal
boolean $doMin
getobjinfo (boundsChecking[], $setMin; info=dataElement, dim1=setMin)
getobjinfo (boundsChecking[], $minVal; info=dataElement, dim1=minVal)
if ($setMin == 1)
	$doMin = true
endif

integer $doRenorm, $normDimNum
getobjinfo (normInfo[], $doRenorm; info=dataElement, dim1=doRenorm)
getobjinfo (normInfo[], $normDimNum; info=dataElement, dim1=normDimNum)
if ($doRenorm)
	$doMax = true
	$maxVal = 1.0
	$doMin = true
	$minVal = 0.0
endif

real $percentChgFromLYHis
getobjinfo (percentChgFromLYHis[], $percentChgFromLYHis; info=dataElement)

integer $defaultJoin
getobjinfo (defaultJoin[], $defaultJoin; info=dataElement)

say ("Doing saturating linear trend")

boolean $nonDefault
say ("")
say ("Default setting are:")
say ("  first historical time for basing trends = ", $defaultYearToStartProj)
say ("  percent change from last year of history = ", $percentChgFromLYHis)
say ("")
say ("  You can choose to have the projection either join the;")
say ("    1) projection of the last 2 years of history")
say ("    2) projection of the trend line")
say ("    3) projection of the last history year along the trend line")
say ("  The default is: ", $defaultJoin)
say ("")
say ("")

listen ("Do you want to change these values? {y n}[n] ",$nonDefault)
integer $joinMethod		
real $percentChange
local percentChangex[]
if $nonDefault == false
	percentChangex[] = create (; data=$percentChgFromLYHis)
	$firstHisYear = $defaultYearToStartProj
	$joinMethod = $defaultJoin
else
	say ("")
	say ("What is historical starting year for the projection?{", $minYearToStartProj, "..", $maxYearToStartProj, "}[", $defaultYearToStartProj, "] ")
	listen (" ", $firstHisYear)
	if $firstHisYear == 0
		$firstHisYear = $defaultYearToStartProj
	endif
	if $firstHisYear < $minYearToStartProj AND ($firstHisYear != 0)
		$firstHisYear = $minYearToStartProj
	endif
	if $firstHisYear > $maxYearToStartProj
		$firstHisYear = $maxYearToStartProj
	endif

	say ("What percentage change from last year of history do you want to allow? {0..}[", $percentChgFromLYHis, "]")
	listen (" ", $percentChange)
	if $percentChange == ""
		percentChangex[] = create (; data=$percentChgFromLYHis)
	else
		percentChangex[] = create (; data=$percentChange)
	endif

	$joinMethod = $defaultJoin
	boolean $joinHisToProjectionMethod[3]
	menu ("What method do you want to use to join the projection to history?", 3, \
		"projection of the last 2 years of history", \
		"projection of the trend line", \
		"projection of the last history year along the trend line", \
		$joinHisToProjectionMethod[], defaultChoice=$defaultJoin, \
		oneChoice=true)
	if $joinHisToProjectionMethod[1] == true
		$joinMethod == 1
	endif
	if $joinHisToProjectionMethod[2] == true
		$joinMethod == 2
	endif
	if $joinHisToProjectionMethod[3] == true
		$joinMethod == 3
	endif
endif

firstHisYearForProjection[] = create (; dataType=integer , data=$firstHisYear)
joinMethod[] = create (; dataType=integer, data=$joinMethod)
asymtoticPercentChange[] = percentChangex[]

local variableHis[upperDims,t] = varNameHis[upperdims,timeseq]

local trendx[upperdims,timeseq], projx[upperdims,timeseq]
if ($joinMethod == 1) or ($joinMethod == 3)
	trendx[upperdims,timeseq], projx[upperdims,timeseq] = lintrend (extract (variableHis[upperDims,t]; \
		time:$firstHisYear..-0); time=$lastScnYear, join=on)
endif
if $joinMethod == 2
	trendx[upperdims,timeseq], projx[upperdims,timeseq] = lintrend (extract (variableHis[upperDims,t]; \
		time:$firstHisYear..-0); time=$lastScnYear)
endif

local trendy[upperdims,timeseq], projy[upperdims,timeseq]
if ($doMax == true)
	trendy[upperdims,timeseq] = min (trendx[upperdims,timeseq], $maxVal)
	trendx[upperdims,timeseq] = trendy[upperdims,timeseq]
	projy[upperdims,timeseq] = min (projx[upperdims,timeseq], $maxVal)
	projx[upperdims,timeseq] = projy[upperdims,timeseq]
endif
if ($doMin == true)
	trendy[upperdims,timeseq] = max (trendx[upperdims,timeseq], $minVal)
	trendx[upperdims,timeseq] = trendy[upperdims,timeseq]
	projy[upperdims,timeseq] = max (projx[upperdims,timeseq], $minVal)
	projx[upperdims,timeseq] = projy[upperdims,timeseq]
endif
if ($doRenorm == 1)
	trendy[upperdims,timeseq] = norm (trendx[upperdims,timeseq]; dim=$normDimNum)
	trendx[upperdims,timeseq] = trendy[upperdims,timeseq]
	projy[upperdims,timeseq] = norm (projx[upperdims,timeseq]; dim=$normDimNum)
	projx[upperdims,timeseq] = projy[upperdims,timeseq]
endif

local trend[upperdims,timeseq] = trendx[upperdims,timeseq]
local proj[upperdims,timeseq] = projx[upperdims,timeseq]


local trendp[upperDims] = extract (delta (trend[upperdims,timeseq]); time:-0, shrink=on)
local positive[mac->aft,ael] = boolgt (trendp[upperDims], 0)
local negative[mac->aft,ael] = boolle (trendp[upperDims], 0)

local percentChange[] = max (percentChangex[], 0)

local firstProjPoint[mac->aft,ael] = extract (proj[upperdims,timeseq]; time:+0, shrink=on)

local asymtotepositive[mac->aft,ael]
if ($doMax == true)
	asymtotepositive[mac->aft,ael] = min (percentChange[] / 100 + 1 * firstProjPoint[upperDims] * positive[mac->aft,ael], $maxVal)
else 
	asymtotepositive[mac->aft,ael] = percentChange[] / 100 + 1 * firstProjPoint[upperDims] * positive[mac->aft,ael]
endif 
	
local asymtotenegative[mac->aft,ael]
if ($doMin == true)
	asymtotenegative[mac->aft,ael] = max (percentChange[] / 100 * -1 + 1 * firstProjPoint[upperDims] * negative[mac->aft,ael], $minVal)
else
	asymtotenegative[mac->aft,ael] = percentChange[] / 100 * -1 + 1 * firstProjPoint[upperDims] * negative[mac->aft,ael]
endif
	
local asymtote[mac->aft,ael] = asymtotepositive[upperDims] + asymtotenegative[upperDims]

local base[cr,dts,bt] = extract (trend[upperdims,timeseq]; time:-1..-0)
local satprojx[upperdims,timeseq] = satproj (base[cr,dts,bt], asymtote[upperDims]; time=$lastScnYear)

local satprojy[upperdims,timeseq]
if ($doMax == true)
	satprojy[upperdims,timeseq] = min (satprojx[upperdims,timeseq], $maxVal)
	satprojx[upperdims,timeseq] = satprojy[upperdims,timeseq]
endif
if ($doMin == true)
	satprojy[upperdims,timeseq] = max (satprojx[upperdims,timeseq], $minVal)
	satprojx[upperdims,timeseq] = satprojy[upperdims,timeseq]
endif
if ($doRenorm == 1)
	satprojy[upperdims,timeseq] = norm (satprojx[upperdims,timeseq]; dim=$normDimNum)
	satprojx[upperdims,timeseq] = satprojy[upperdims,timeseq]
endif	
local satprojection[upperdims,timeseq] = 	satprojx[upperdims,timeseq]

local projFirstPoint[upperDims] = extract (satprojection[upperdims,timeseq]; time:+0, shrink=on)

local trendFLPx[upperdims,timeseq], projFLPx[upperdims,timeseq]

if $joinMethod == 1
	trendFLPx[upperdims,timeseq], projFLPx[upperdims,timeseq] = lintrend (extract (variableHis[upperDims,t]; \
		time:-1..-0); time=$lastHisYearP1, join=on)
endif
if $joinMethod == 2
	trendFLPx[upperdims,timeseq], projFLPx[upperdims,timeseq] = lintrend (extract (trend[upperdims,timeseq]; \
		time:-1..-0); time=$lastHisYearP1)
endif
if $joinMethod == 3
	trendFLPx[upperdims,timeseq], projFLPx[upperdims,timeseq] = lintrend (extract (trend[upperdims,timeseq]; \
		time:-1..-0); time=$lastHisYearP1)
	local lastYearOfTrend[upperDims] = extract (trend[upperdims,timeseq]; time:-0, shrink=on)
	local secondLastYearOfTrend[upperDims] = extract (trend[upperdims,timeseq]; time:-1, shrink=on)
	local lastYearOfHistory[upperDims] = extract (variableHis[upperDims,t]; time:-0, shrink=on)
	local firstYearOfProjectionTarget[upperDims] = lastYearOfTrend[upperDims] / secondLastYearOfTrend[upperDims] * lastYearOfHistory[upperDims]
	local firstYearOfProjection[upperDims] = extract (projFLPx[upperdims,timeseq]; time:+0, shrink=on)
	local projFLPxx[upperdims,timeseq] = firstYearOfProjectionTarget[upperDims] / firstYearOfProjection[upperDims]  * projFLPx[upperdims,timeseq]
	projFLPx[upperdims,timeseq] = projFLPxx[upperdims,timeseq]
endif

! Adjust both trend and proj so that if they went to 0 they stayed there!
local trendFLPy[upperdims,timeseq], projFLPy[upperdims,timeseq]
if ($doMax == true)
	trendFLPy[upperdims,timeseq] = min (trendFLPx[upperdims,timeseq], $maxVal)
	trendFLPx[upperdims,timeseq] = trendFLPy[upperdims,timeseq]
	projFLPy[upperdims,timeseq] = min (projFLPx[upperdims,timeseq], $maxVal)
	projFLPx[upperdims,timeseq] = projFLPy[upperdims,timeseq]
endif
if ($doMin == true)
	trendFLPy[upperdims,timeseq] = max (trendFLPx[upperdims,timeseq], $minVal)
	trendFLPx[upperdims,timeseq] = trendFLPy[upperdims,timeseq]
	projFLPy[upperdims,timeseq] = max (projFLPx[upperdims,timeseq], $minVal)
	projFLPx[upperdims,timeseq] = projFLPy[upperdims,timeseq]
endif
if ($doRenorm == 1)
	trendFLPy[upperdims,timeseq] = norm (trendFLPx[upperdims,timeseq]; dim=$normDimNum)
	trendFLPx[upperdims,timeseq] = trendFLPy[upperdims,timeseq]
	projFLPy[upperdims,timeseq] = norm (projFLPx[upperdims,timeseq]; dim=$normDimNum)
	projFLPx[upperdims,timeseq] = projFLPy[upperdims,timeseq]
endif	
local trendFLP[upperdims,timeseq] = trendFLPx[upperdims,timeseq]
local projFLP[upperdims,timeseq] = projFLPx[upperdims,timeseq]

local projFLPz[upperDims] = shrink (projFLP[upperdims,timeseq])

local adjust[upperDims] = projFLPz[upperDims] / projFirstPoint[upperDims]

local varNamex[upperdims,timeseq] = satprojection[upperdims,timeseq] * adjust[upperDims]
	
local varNamey[upperdims,timeseq]

if ($doMax == true)
say ("processing do max in subroutine")
	varNamey[upperdims,timeseq] = min (varNamex[upperdims,timeseq], $maxVal)
	varNamex[upperdims,timeseq] = varNamey[upperdims,timeseq]
endif

if ($doMin == true)
say ("processing do min in subroutine")
	varNamey[upperdims,timeseq] = max (varNamex[upperdims,timeseq], $minVal)
	varNamex[upperdims,timeseq] = varNamey[upperdims,timeseq]
endif

if ($doRenorm == 1)
say ("processing do renormilization in subroutine")
	varNamey[upperdims,timeseq] = norm (varNamex[upperdims,timeseq]; dim=$normDimNum)
	varNamex[upperdims,timeseq] = varNamey[upperdims,timeseq]
endif	
varNameResult[upperdims,timeseq] = varNamex[upperdims,timeseq]


boolean $seeResults
$seeResults = false
!say ("")
!listen ("Do you want to see the results? ",$seeResults)
if $seeResults == true
	graph (trend[upperdims,timeseq], varNameHis[upperdims,timeseq], varNameResult[upperdims,timeseq]; \
		objColours=$scnColours[all], \
		title="trend, historical, background and new intensity")
endif

